///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//    Copyright (c) 2019 AutoNavi Software Co.,Ltd. All rights reserved.     //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////
#pragma once

#include <inttypes.h>
#include <stddef.h>

namespace ALinkClientSDK {

#define API_PUBLIC __attribute__ ((visibility ("default")))

enum API_PUBLIC EALINKKEYCODE {
    EALINK_KEYCODE_PLAY = 0x0000007E,  // |
    EALINK_KEYCODE_PAUSE = 0x0000007F,  // ||
    EALINK_KEYCODE_PREV = 0x00000058,  // <<
    EALINK_KEYCODE_NEXT = 0x00000057,  // >>
    EALINK_KEYCODE_KNOBS_LEFT = 0x00000060,  // Rotate/left
    EALINK_KEYCODE_KNOBS_RIGHT = 0x00000061,  // Rotate/right
    EALINK_KEYCODE_KNOBS_MIDDLE = 0x00000062,  // push
    EALINK_KEYCODE_ASR_STOP_PAL = 0x00000056,  // asr stop
    EALINK_KEYCODE_ASR_START_PAL = 0x00000115,  // asr start
};

enum API_PUBLIC EALINKBUTTONSTATE {
    EALINK_BUTTONSTATE_PRESSED = 0x0000000B,
    EALINK_BUTTONSTATE_RELEASED = 0x0000000C,
};

enum API_PUBLIC EALINKTOUCHACTION {
    EALINK_TOUCH_DOWN = 0,
    EALINK_TOUCH_UP,
    EALINK_TOUCH_MOVE,
};

enum API_PUBLIC EALINKHMITYPE {
    EALINK_KEY_EVENT = 1,
    EALINK_TOUCH_EVENT = 3,
};

enum API_PUBLIC EALINKSTREAMTYPE {
    EALINK_VIDEO = 0,
    EALINK_MUSIC,
    EALINK_NAVIGATION,
    EALINK_SPEECH,
};

enum API_PUBLIC EALINKNOTIFICATIONTYPE {
    EALINK_NOTIFICATION_CONNECT_STATUS      = 0,  // ALinkServer连接状态
    EALINK_NOTIFICATION_VIDEO               = 1,  // 视频状态
    EALINK_NOTIFICATION_MUSIC               = 2,  // 暂时不用
    EALINK_NOTIFICATION_NAVI                = 3,  // 进入或者退出导航模式
    EALINK_NOTIFICATION_SPEECH              = 4,  // 车盒收音打开或关闭
    EALINK_NOTIFICATION_MIC                 = 5,  // 暂时不用
    // 多媒体透传信息
    EALINK_NOTIFICATION_MEDIA_TRACKDATA     = 6,  // 曲目信息
    EALINK_NOTIFICATION_MEDIA_STATUS        = 7,  // 音乐播放开始、停止、暂停、快进（暂时不用）、快退（暂时不用）
    EALINK_NOTIFICATION_MEDIA_ELAPSETIME    = 8,  // 歌曲播放时间

    EALINK_NOTIFICATION_RESOURCES_OWNERSHIP = 9,  // 暂时不用
};

typedef struct API_PUBLIC {
    int32_t button;       // EALINKKEYCODE
    int32_t buttonState;  // EALINKBUTTONSTATE
} SButtonEvent;

typedef struct API_PUBLIC {
    int32_t x;
    int32_t y;
    int32_t slot;  // defaultValue = -1 (means the parameter can not use)
} SPointer;

typedef struct API_PUBLIC {
    int32_t action;  // 0:down 1:up 2:move
    int32_t count;  // 实际有效点个数<=5
    SPointer pointers[5];
} STouchEvent;

typedef struct API_PUBLIC {
    int32_t type;  // 1: key  3: touch
    union {
        SButtonEvent* buttonEvent;
        STouchEvent* touchEvent;
    };
} SHMIEvent;

class API_PUBLIC IALinkClientAVListener {
 public:
    /**
     * ALink音视频数据更新时的回调接口。
     *
     * @param data 音视频数据
     * @param size 音视频数据长度
     * @param timeStamp 时间戳
     * @param marker 数据包结束时置true，否则为false
     */
    virtual int32_t onAVDataReceived(const uint8_t* data, size_t size, size_t timeStamp, bool marker = false) = 0;
};

class API_PUBLIC IALinkClientEventListener {
 public:
    /**
     * ALink事件回调接口。
     *
     * @param msg 事件内容. 其中,event是NOTIFICATION_TYPE中的内容
     *
     * 与ALinkServer连接状态：{"event":0,status":0} status：0连接上，1断开连接
     * video状态:            {"event":1,status":0} status：0开始，1停止
     * music状态:            {"event":2,status":0} status：0开始，1停止
     * navi状态:             {"event":3,status":0} status：0开始，1停止
     * speech状态:           {"event":4,status":0} status：0开始，1停止
     * mic状态:              {"event":5,status":0} status：0开始，1停止
     *
     * 多媒体透传信息
     * trackdata曲目信息:     {"event":6,source":"music/FM/ABook","title":"歌曲名称","duration":歌曲长度,"album":"专辑","artist":"作者"}
     * playstate音乐播放状态:  {"event":7,status":0} status：0开始、1停止、2暂停、3快进（暂时不用）、4快退（暂时不用）
     * elapsetime播放时间:    {"event":8,elapsetime":1} elapsetime:已播放时间(单位是秒)
     *
     * @param size 数据长度
     */
    virtual int32_t onEventReceived(const char* msg, size_t size) = 0;
};


class API_PUBLIC ALinkSDK {
 public:
     ALinkSDK();
     ~ALinkSDK();

    /**
     * 初始化ALINKSDK。
     *
     * @param param 配置参数，param为json，格式如下：
     * {
     *  "logLevel":0,                                         // 日志级别0-8
     *  "logPath":"/data/",                                   // 日志路径
     *  "verifyInfoPath":"/data/alink/certs/ca_chain.pem",    // 证书文件路径
     *  "certificatePath":"/data/alink/certs/devicecert.pem", // 证书文件路径
     *  "privateKeyPath":"/data/alink/certs/devicekey.pem"    // 证书文件路径
     *  "codecWidth":800,                                     // 客户端编码长度
     *  "codecHeight":480,                                    // 客户端编码宽度
     *  "frameRate":30,                                       // 帧率
     *  "dpi":120                                             // 每英寸点数
     *  "audioNumChannels_media":2,                           // media通道数
     *  "audioSampleRate_media":48000,                        // media采样率
     *  "audioBitDepth_media":16,                             // media比特位
     *  "audioNumChannels_speech":1,                          // speech通道数
     *  "audioSampleRate_speech":16000,                       // speech采样率
     *  "audioBitDepth_speech":16,                            // speech比特位
     *  "audioNumChannels_navigation":1,                      // navi通道数
     *  "audioSampleRate_navigation":16000,                   // navi采样率
     *  "audioBitDepth_navigation":16,                        // navi比特位
     * }
     * @param size 数据长度
     *
     */
    void init(const char* param, size_t size);

    /**
     * 动态设置服务端音视频参数。
     *
     * @param param 配置参数，param为json，格式如下：
     * {
     *  "codecWidth":800,                                     // 服务端编码长度
     *  "codecHeight":480,                                    // 服务端编码宽度
     *  "frameRate":30,                                       // 帧率
     *  "dpi":120                                             // 每英寸点数
     *  "audioNumChannels_media":2,                           // media通道数
     *  "audioSampleRate_media":48000,                        // media采样率
     *  "audioBitDepth_media":16,                             // media比特位
     *  "audioIPL_media":4800,                                // media的IPL
     *  "audioNumChannels_speech":1,                          // speech通道数
     *  "audioSampleRate_speech":16000,                       // speech采样率
     *  "audioBitDepth_speech":16,                            // speech比特位
     *  "audioIPL_speech":1600,                               // speech的IPL
     *  "audioNumChannels_navigation":1,                      // navi通道数
     *  "audioSampleRate_navigation":16000,                   // navi采样率
     *  "audioBitDepth_navigation":16,                        // navi比特位
     *  "audioIPL_navigation":1600,                           // navi的IPL
     * }
     * @param size 数据长度
     *
     * 备注：必须CLient端与Server端建立连接此接口才生效，其中音频参数和视频参数可以分开设置。
     */
    void setParameter(const char* param, size_t size);

    /**
     * 获取服务端音视频参数。
     *
     * @param param 配置参数，param为json，格式如下：
     * {
     *  "codecWidth":800,                                     // 服务端编码长度
     *  "codecHeight":480,                                    // 服务端编码宽度
     *  "frameRate":30,                                       // 帧率
     *  "dpi":120                                             // 每英寸点数
     *  "audioNumChannels_media":2,                           // media通道数
     *  "audioSampleRate_media":48000,                        // media采样率
     *  "audioBitDepth_media":16,                             // media比特位
     *  "audioIPL_media":4800,                                // media的IPL
     *  "audioNumChannels_speech":1,                          // speech通道数
     *  "audioSampleRate_speech":16000,                       // speech采样率
     *  "audioBitDepth_speech":16,                            // speech比特位
     *  "audioIPL_speech":1600,                               // speech的IPL
     *  "audioNumChannels_navigation":1,                      // navi通道数
     *  "audioSampleRate_navigation":16000,                   // navi采样率
     *  "audioBitDepth_navigation":16,                        // navi比特位
     *  "audioIPL_navigation":1600,                           // navi的IPL
     * }
     * @param size 数据长度
     *
     * 备注：必须CLient端与Server端建立连接此接口才生效。
     */
    void getParameter(char* param, size_t size);

    /**
     * 向SDK注册音视频流接口。
     *
     * @param type 音视频流类型
     *
     * @param listener 音视频流接口
     * listener设置为NULL时进行解注册。
     *
     * 备注：listener的生命周期至少不比ALinkSDK短，一种类型仅限一个listener，
     * 非线程安全，最好与ALinkSDK对象在同一个线程。
     */
    void registerAVListener(EALINKSTREAMTYPE type, IALinkClientAVListener* listener);

    /**
     * 向ALINKSDK注册消息通知接口。
     *
     * @param handler 消息通知接口
     * listener设置为NULL时进行解注册
     *
     * 备注：listener的生命周期至少不比ALinkSDK短，仅限一个事件listener。
     * 非线程安全，最好与ALinkSDK对象在同一个线程。
     */
    void registerEventListener(IALinkClientEventListener* listener);

    /**
     * 启动SDK。
     *
     */
    void start();

    /**
     * 停止SDK。
     *
     */
    void stop();

    /**
     * 车机音视频流通道建立或断开时给ALINKSDK发送通知消息。
     *
     * @param msg 消息内容，msg为json。
     *
     * 车机视频流通道建立时给ALINKSDK发送消息格式：
     * {"status":"video-start"}
     * video-start与video-ready均可，推荐使用audio-start
     *
     * 车机视频流通道关闭时给ALINKSDK发送消息格式：
     * {"status":"video-stop"}
     *
     * 车机音频流通道建立时给ALINKSDK发送消息格式：
     * {"status":"audio-start"}
     * 备注：audio-start与audio-ready均可，推荐使用audio-start
     *
     * 车机音频流通道关闭时给ALINKSDK发送消息格式:
     * {"status":"audio-stop"}
     *
     * @param size 数据长度
     *
     * @return true：成功，false：失败。
     */
    bool postCommandMessage(const char* msg, size_t size);

    /**
     * 车机向ALINKSDK发送方控、旋钮、触屏事件。
     *
     * @param hmiEvent 消息内容。
     *
     * @return true：成功，false：失败。
     */
    bool postHMIEvent(const SHMIEvent & hmiEvent);

    /**
     * 车机向ALINKSDK发送麦克风数据。
     *
     * @param data 数据内容
     * @param size 数据长度
     * @return true：成功，false：失败。
     */
    bool postMicStream(const uint8_t* data, size_t size);

    /**
     * 获取SDK版本号。
     *
     * @return SDK版本号。
     */
    const char* getVersion();
};
}  // namespace ALinkClientSDK

